/**	\file Types.cpp
 **	\date  2008-05-26
 **	\author grymse@alhem.net
**/
/*
Copyright (C) 2008-2009  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "Types.h"
#include "Wsdl.h"
#include "Application.h"
#include "WsdlException.h"
#include <Sockets/Debug.h>


Types::Types(const Application& appl, const Wsdl& wsdl)
: m_appl(appl)
, m_wsdl(wsdl)
{
}


Types::~Types()
{
}


void Types::display(const std::string& name, const XmlNode& node)
{
	printf("    %s  %s", name.c_str(), node.GetProperty("type").c_str());
	if (node.PropertyExists("minOccurs"))
		printf("  minOccurs %s", node.GetProperty("minOccurs").c_str());
	if (node.PropertyExists("maxOccurs"))
		printf("  maxOccurs %s", node.GetProperty("maxOccurs").c_str());

	//
	printf("\n");
}


void Types::generate()
{
	Debug deb("generate()");
	std::list<std::string> ops = m_wsdl.GetOperations( m_appl.Portname() );

	for (std::list<std::string>::iterator it = ops.begin(); it != ops.end(); it++)
	{
		std::string op = *it;

		printf("Operation: %s\n", op.c_str());
		printf("SOAPaction: %s\n", m_wsdl.GetSoapAction(m_appl.Portname(), op).c_str());

		std::string in;
		std::string out;
		std::string path;
		try {
			in = m_wsdl.GetMessageForOp(m_appl.Portname(), op, "input");
		} catch (const WsdlException&) {}
		if (!in.empty())
		{
			std::list<std::pair<std::string, XmlNode> > vec = m_wsdl.GetMessageParameters(in, path);
			printf("Input: %s\nPath: %s\n", in.c_str(), path.c_str());
			for (std::list<std::pair<std::string, XmlNode> >::iterator it = vec.begin(); it != vec.end(); it++)
			{
				display(it -> first, it -> second);
			}
		}
		try {
			out = m_wsdl.GetMessageForOp(m_appl.Portname(), op, "output");
		} catch (const WsdlException&) {}
		if (!out.empty())
		{
			std::list<std::pair<std::string, XmlNode> > vec = m_wsdl.GetMessageParameters(out, path);
			printf("Output: %s\nPath: %s\n", out.c_str(), path.c_str());
			for (std::list<std::pair<std::string, XmlNode> >::iterator it = vec.begin(); it != vec.end(); it++)
			{
				display(it -> first, it -> second);
			}
		}
		//
		printf("\n");
	}
}


